# -*- coding: utf-8 -*-
# Copyright (c) 2021, Havenir Solutions and contributors
# For license information, please see license.txt

from __future__ import unicode_literals
import frappe, math, json
from frappe import _
import erpnext
from frappe.model.document import Document
from frappe.utils import flt, rounded, add_months, nowdate, getdate,get_datetime
from erpnext.controllers.accounts_controller import AccountsController
from frappe.model.mapper import get_mapped_doc

class Reservations(Document):

	def validate(self):
		validate_repayment_method(self.repayment_method, self.amount, self.monthly_repayment_amount, self.repayment_periods)
		self.set_missing_fields()
		if self.reservation_type=='Selling':
			self.make_repayment_schedule()
			self.set_repayment_period()

	def on_submit(self):
		if(get_datetime() < get_datetime(self.reservation_end)):
			frappe.db.set_value("Unit", self.unit, "status", "Reserved")
		else:
			frappe.db.set_value("Unit", self.unit, "status", "Vacant")

	def on_cancel(self):
		frappe.db.set_value("Unit", self.unit, "status", "Vacant")

	def set_missing_fields(self):
		if self.repayment_method == "Repay Over Number of Periods":
			self.monthly_repayment_amount = get_monthly_repayment_amount(self.repayment_method, self.amount, self.rate_of_interest, self.repayment_periods,self.advance_payment)

	def set_repayment_period(self):
		if self.repayment_method == "Repay Fixed Amount per Period":
			repayment_periods = len(self.repayment_schedule)
			self.repayment_periods = repayment_periods

	def make_repayment_schedule(self):
		self.repayment_schedule = []

		payment_date = self.repayment_start_date
		balance_amount = self.amount - self.advance_payment
		while(balance_amount > 0):
			interest_amount = (balance_amount * flt(self.rate_of_interest) / (12*100))
			principal_amount = self.monthly_repayment_amount - interest_amount
			balance_amount = (balance_amount + interest_amount - self.monthly_repayment_amount)

			if balance_amount < 0:
				principal_amount += balance_amount
				balance_amount = 0.0

			total_payment = principal_amount + interest_amount
			self.append("repayment_schedule", {
				"payment_date": payment_date,
				"principal_amount": principal_amount,
				"interest_amount": interest_amount,
				"total_payment": total_payment,
				"balance_amount": balance_amount
			})
			next_payment_date = add_months(payment_date, 1)
			payment_date = next_payment_date

def validate_repayment_method(repayment_method, amount, monthly_repayment_amount, repayment_periods):
	if repayment_method == "Repay Over Number of Periods" and not repayment_periods:
		frappe.throw(_("Please enter Repayment Periods"))

	if repayment_method == "Repay Fixed Amount per Period":
		if not monthly_repayment_amount:
			frappe.throw(_("Please enter repayment Amount"))
		if monthly_repayment_amount > amount:
			frappe.throw(_("Monthly Repayment Amount cannot be greater than Amount"))

def get_monthly_repayment_amount(repayment_method, amount, rate_of_interest, repayment_periods,advance_payment):
	if rate_of_interest:
		monthly_interest_rate = flt(rate_of_interest) / (12 *100)
		monthly_repayment_amount = math.ceil(((amount-advance_payment) * monthly_interest_rate *
			(1 + monthly_interest_rate)**repayment_periods) \
			/ ((1 + monthly_interest_rate)**repayment_periods - 1))
	else:
		monthly_repayment_amount = math.ceil(flt((amount-advance_payment)) / repayment_periods)
		# frappe.msgprint(str(repayment_periods))
	return monthly_repayment_amount



@frappe.whitelist()
def make_unit_sale_contract(source_name, target_doc=None):
	doclist = get_mapped_doc("Reservations", source_name, {
		"Reservations": {
			"doctype": "Unit Sale Contract"
		},
		"Reservation Repayment Schedule": {
			"doctype": "Reservation Repayment Schedule",
			"field_map": [
				["payment_date", "payment_date"],
				["principal_amount", "principal_amount"],
				["interest_amount", "interest_amount"],
				["total_payment", "total_payment"],
				["balance_amount", "balance_amount"]
			]
		}
	}, target_doc)

	return doclist


@frappe.whitelist()
def make_property_contract(source_name, target_doc=None):
	doclist = get_mapped_doc("Reservations", source_name, {
		"Reservations": {
			"doctype": "Property Contract"
		},
		"Reservation Repayment Schedule": {
			"doctype": "Reservation Repayment Schedule",
			"field_map": [
				["payment_date", "payment_date"],
				["principal_amount", "principal_amount"],
				["interest_amount", "interest_amount"],
				["total_payment", "total_payment"],
				["balance_amount", "balance_amount"]
			]
		}
	}, target_doc)

	return doclist